import os
import shutil
from lib import paths_management as paths

app_paths = paths.App_Paths()
db_path = os.path.join(app_paths.base_script_path, 'web_downloader.db')

if os.path.isfile(db_path):
    os.remove(db_path)
if os.path.isdir('web_data'):
    shutil.rmtree('web_data')
if os.path.isdir('firefox_driver'):
    shutil.rmtree('firefox_driver')


import os

class App_Paths:
    path_module_script_path = os.path.abspath(__file__)
    base_script_path = os.path.dirname(os.path.dirname(path_module_script_path))
    base_web_data_dir_path = os.path.join(base_script_path, 'web_data')
    base_firefox_driver_path = os.path.join(base_script_path, 'firefox_driver')
    firefox_driver_path = None

    def __init__(self):
        self.try_set_firefox_driver_path(False)

    def try_set_firefox_driver_path(self, verbose=True):
        if self.firefox_driver_path is None:
            if os.path.isdir(self.base_firefox_driver_path) == False:
                if verbose:
                    print('Firefox driver directory has not yet been created')
                    print(f'Firefox driver directory path: {self.base_firefox_driver_path}')
                return None
            if len(os.listdir(self.base_firefox_driver_path)) == 0:
                if verbose:
                    print('Firefox driver directory is empty. Please download firefox driver')
                    print(f'Firefox driver directory path: {self.base_firefox_driver_path}')
                return None
            self.firefox_driver_path = os.path.join(self.base_firefox_driver_path, os.listdir(self.base_firefox_driver_path)[0], 'geckodriver.exe')
        return self.firefox_driver_path


import os

# import db
from lib import paths_management as paths
from lib import db

app_paths = paths.App_Paths()
db_path = os.path.join(app_paths.base_script_path, 'web_downloader.db')

curr_db = db.Database(db_path)
curr_db.create_db()


from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support.expected_conditions import presence_of_element_located
import pyautogui
import time
from . import db
from . import paths_management as paths
import os

def occupy_url_table_row():
    curr_db = db.Database()
    url_id = curr_db.insert_url('', '', -2, '')
    assert url_id >= 0
    return url_id

def insert_url_to_db(url_id, url, title, depth_level, file_path):
    curr_db = db.Database()
    curr_db.update_url(url_id, url, title, depth_level, file_path)

def download_url(url, curr_depth_level, wait_time=5, retry=2):
    app_paths = paths.App_Paths()

    # Create directory if the firefox driver directory is not created
    if not os.path.exists(app_paths.base_firefox_driver_path):
        os.mkdir(app_paths.base_firefox_driver_path)

    # Execute the script to download Firefox driver
    from . import get_firefox_driver as webdriver_getter
    if len(os.listdir(app_paths.base_firefox_driver_path)) == 0:
        webdriver_getter.main()

    assert app_paths.try_set_firefox_driver_path() is not None
    
    print('Using Firefox driver from:')
    print(app_paths.firefox_driver_path)

    # Create web data directory
    if not os.path.isdir(app_paths.base_web_data_dir_path):
        os.mkdir(app_paths.base_web_data_dir_path)

    url_id = occupy_url_table_row()

    page_title = '__blank__'
    file_path = os.path.join(app_paths.base_web_data_dir_path, f'{url_id}.html')

    while True:
        if os.path.isfile(file_path):
            break
        if retry == 0:
            print('Exceeded retry')
            break

        with webdriver.Firefox(executable_path=app_paths.firefox_driver_path) as driver:
            driver.get(url)
            time.sleep(wait_time)
            page_title = driver.title
            pyautogui.hotkey('ctrl', 's')
            time.sleep(1)
            
            pyautogui.typewrite(file_path)
            time.sleep(2)
            pyautogui.hotkey('enter')
            time.sleep(5)
        retry -= 1

    insert_url_to_db(url_id, url, page_title, curr_depth_level, file_path)

    return url_id